using System;
using System.Collections.Generic;
using System.Collections;

using System.Reflection;
using System.Drawing;
using log4net;
using Nini.Config;
using OpenMetaverse;

using OpenSim.Framework;
using OpenSim.Data;
using OpenSim.Region.Environment;
using OpenSim.Region.Environment.Interfaces;
using OpenSim.Region.Environment.Scenes;

namespace Guru {
    public class NagiosModule : IRegionModule {

        private static readonly ILog m_log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);
        private Dictionary<string,SceneObjectGroup> server = new Dictionary<string,SceneObjectGroup>();
        private CSNagios csn = new CSNagios();
        private Uri uri = null;
        private string username = null;
        private string password = null;


        List<Scene> m_scenes = new List<Scene>();
        Dictionary<Scene, List<SceneObjectGroup>> scene_prims = new Dictionary<Scene, List<SceneObjectGroup>>();

        int counter = 0;

        #region IRegionModule interface
        public void Initialise(Scene scene, IConfigSource config) {
            m_log.Info("[GURU] Initializing...");
            
            IniFile ini = new IniFile("nagios_connection.ini");
            uri =  new Uri( ini.ParseFileReadValue( "hostname" ));
            username = ini.ParseFileReadValue("username");
            password = ini.ParseFileReadValue("password");


            m_scenes.Add(scene);
        }

        public void PostInitialise() {
            m_scenes[0].EventManager.OnFrame += new EventManager.OnFrameDelegate(OnTick);
            foreach (Scene s in m_scenes)
                DoHelloWorld(s);
        }

        public void Close() {
        }

        public string Name {
            get { return "Guru Nagios Module"; }
        }

        public bool IsSharedModule {
            get { return true; }
        }

        #endregion

        void DoHelloWorld(Scene scene) {
            // Draw a podest
            List<SceneObjectGroup> prims = new List<SceneObjectGroup>();
            prims.Add( foot( 120, 128 ));
            prims.Add( foot( 128, 128 ));
            prims.Add( foot( 128, 120 ));
            prims.Add( foot( 120, 120 ));

            for ( float i=0; i < 10; i++ ) {
                Vector3 pos = new Vector3( 124, 130 - i / 5f, 21+ i/ 5f );
                PrimitiveBaseShape prim = PrimitiveBaseShape.CreateBox();
                prim.Textures = new Primitive.TextureEntry(new UUID("5748decc-f629-461c-9a36-a35a221fe21f"));
                SceneObjectGroup sog = new SceneObjectGroup(UUID.Zero, pos, prim );
                sog.RootPart.Scale = new Vector3(2f, 0.2f, 0.2f);
                Primitive.TextureEntry tex = sog.RootPart.Shape.Textures;
                Color4 texcolor;
                texcolor = tex.DefaultTexture.RGBA;
                texcolor = new Color4( 1.0f, 0.5f, 0, 1.0f );;
                tex.DefaultTexture.RGBA = texcolor;
                prims.Add( sog );
            } 

            Vector3 pos2 = new Vector3(124, 124, 23);
            SceneObjectGroup sog2 = new SceneObjectGroup(UUID.Zero, pos2, PrimitiveBaseShape.CreateBox());
            sog2.RootPart.Scale = new Vector3(8.3f, 8.3f, 0.3f);
            prims.Add(sog2);

            // Get the nagios service names
            ArrayList services = csn.GetNagiosStatus( uri, username, password ); 
            float step = 8f / services.Count;
            float x = 120f;
            // make a block for each service
            foreach( NagiosStatus service in services ) {
                Vector3 pos3 = new Vector3(128f, x, 24f);
                SceneObjectGroup tmp = new SceneObjectGroup(UUID.Zero, pos3, PrimitiveBaseShape.CreateBox());
                tmp.RootPart.Scale = new Vector3( step , 0.3f, 1f);
                tmp.SetText( service.host + ":"+service.service, new Vector3( 1f, 1f, 1f ), 1f);
                prims.Add( tmp );
                server[ service.host + ":"+service.service ] = tmp;
                x = x + step;
            }

            scene_prims.Add(scene, prims);

            foreach (SceneObjectGroup sogr in prims) {
                scene.AddNewSceneObject(sogr, false);
            }
       }

       SceneObjectGroup foot( float x, float y ) {
            Vector3 pos = new Vector3( x, y, 22);
            PrimitiveBaseShape prim = PrimitiveBaseShape.CreateBox();
            prim.Textures = new Primitive.TextureEntry(new UUID("5748decc-f629-461c-9a36-a35a221fe21f"));

            SceneObjectGroup sog = new SceneObjectGroup(UUID.Zero, pos, prim );
            sog.RootPart.Scale = new Vector3(0.3f, 0.3f, 2f);
            Primitive.TextureEntry tex = sog.RootPart.Shape.Textures;
            Color4 texcolor;
            texcolor = tex.DefaultTexture.RGBA;
            texcolor = new Color4( 1.0f, 0.5f, 0, 1.0f );;
            tex.DefaultTexture.RGBA = texcolor;
            return sog;
        }

        void OnTick() {
            if (counter++ % 500 == 0) {
                // Fetch the current service states
                ArrayList services = csn.GetNagiosStatus( uri, username, password ); 

                foreach( NagiosStatus service in services ) {
                    SceneObjectGroup sog = server[ service.host + ":"+service.service ]; 
                    Primitive.TextureEntry tex = sog.RootPart.Shape.Textures;
                    Color4 texcolor;
                    texcolor = tex.DefaultTexture.RGBA;

                    // Change the color of the prims depending on the service state
                    if ( service.status == "OK" ) {
                        texcolor = new Color4( 0.5f, 1f, 0, 1.0f );
                    } else if ( service.status == "WARNING" ) {
                        texcolor = new Color4( 1.0f, 1.0f, 0, 1.0f );
                    } else {
                        texcolor = new Color4( 1.0f, 0.5f, 0, 1.0f );
                    } 
                    tex.DefaultTexture.RGBA = texcolor;
                    sog.RootPart.UpdateTexture(tex);
                    sog.ScheduleGroupForTerseUpdate();
                }
            }
        }
    }
}
